<?php

namespace App\Http\Controllers\Api\V1;

use App\Actions\User\CreateUser;
use App\Actions\User\DeleteUser;
use App\Actions\User\UpdateUser;
use App\Http\Controllers\Controller;
use App\Http\Requests\AgencyRequest;
use App\Http\Requests\UserRequest;
use App\Http\Resources\AgencyResource;
use App\Models\Agency;
use Illuminate\Http\JsonResponse;

class AgencyController extends Controller
{
    protected array $relations = [
        'users', 'partnerships'
    ];

    public function __construct(
        protected CreateUser $createUser,
        protected UpdateUser $updateUser,
        protected DeleteUser $deleteUser,
    ) {}

    /**
     * Display a listing of the agencies.
     */
    public function index(): JsonResponse {
        $agencies = $this->paginateOrGet(
            Agency::filter()->with('users')->latest());

        return $this->respondWithResourceCollection(
            AgencyResource::collection($agencies)
        );
    }

    /**
     * Store a newly created agency in storage.
     */
    public function store(UserRequest $userRequest, AgencyRequest $agencyRequest): JsonResponse {
        // using database transaction
        $agency = $this->transactional(function () use ($userRequest, $agencyRequest) {
            $agency = Agency::create($agencyRequest->validated());
            $this->createUser->agency($userRequest, $agency->id);

            return $agency;
        });

        return $this->respondCreated(
            new AgencyResource($agency),
            "Agency created successfully!"
        );
    }

    /**
     * Display the specified agency.
     */
    public function show(Agency $agency): JsonResponse {
        $agency->loadMissing($this->relations);

        return $this->respondWithResource(new AgencyResource($agency));
    }

    /**
     * Update the specified agency in storage.
     */
    public function update(UserRequest $userRequest, AgencyRequest $agencyRequest, Agency $agency): JsonResponse {
        // using database transaction
        $this->transactional(function () use ($userRequest, $agencyRequest, $agency) {
            $this->updateUser->handle($userRequest, $agency->users()->findOrFail($userRequest->validated("user_id")));
            $agency->update($agencyRequest->validated());
        });

        return $this->respondUpdated(
            new AgencyResource($agency),
            "Agency updated successfully!"
        );
    }

    /**
     * Remove the specified agency from storage.
     */
    public function destroy(Agency $agency): JsonResponse {
        $agency->delete();

        return $this->respondSuccess("Agency deleted successfully!");
    }

    public function storeUser(UserRequest $userRequest, Agency $agency) {
        $this->createUser->agency($userRequest, agencyId: $agency->id);

        return $this->respondCreated(
            new AgencyResource($agency),
            'Agency user created successfully!'
        );
    }
}
