<?php

namespace App\Http\Controllers\Api\V1;

use App\Actions\User\CreateUser;
use App\Actions\User\DeleteUser;
use App\Actions\User\UpdateUser;
use App\Http\Controllers\Controller;
use App\Http\Requests\HirerRequest;
use App\Http\Requests\UserRequest;
use App\Http\Resources\HirerResource;
use App\Models\Hirer;
use Illuminate\Http\JsonResponse;

class HirerController extends Controller
{
    protected array $relations = [
        "user"
    ];

    public function __construct(
        protected CreateUser $createUser,
        protected UpdateUser $updateUser,
        protected DeleteUser $deleteUser,
    ) {}

    /**
     * Display a listing of the hirers.
     */
    public function index(): JsonResponse {
        $hirers = $this->paginateOrGet(Hirer::filter()->latest());

        return $this->respondWithResourceCollection(
            HirerResource::collection($hirers)
        );
    }

    /**
     * Store a newly created hirer in storage.
     */
    public function store(UserRequest $userRequest, HirerRequest $hirerRequest): JsonResponse {
        // using database transaction
        $hirer = $this->transactional(function () use ($userRequest, $hirerRequest) {
            $user = $this->createUser->hirer($userRequest);
            return $user->hirer()->create($hirerRequest->validated());
        });

        return $this->respondCreated(
            new HirerResource($hirer->loadMissing('user')),
            "Hirer created successfully!"
        );
    }

    /**
     * Display the specified hirer.
     */
    public function show(Hirer $hirer): JsonResponse {
        $hirer->loadMissing($this->relations);

        return $this->respondWithResource(new HirerResource($hirer));
    }

    /**
     * Update the specified hirer in storage.
     */
    public function update(UserRequest $userRequest, HirerRequest $hirerRequest, Hirer $hirer): JsonResponse {
        // using database transaction
        $this->transactional(function () use ($userRequest, $hirerRequest, $hirer) {
            $this->updateUser->handle($userRequest, $hirer->user);
            return $hirer->update($hirerRequest->validated());
        });

        return $this->respondUpdated(
            new HirerResource($hirer->loadMissing('user')),
            "Hirer updated successfully!"
        );
    }

    /**
     * Remove the specified hirer from storage.
     */
    public function destroy(Hirer $hirer): JsonResponse {
        // hirer will be deleted automatically because of cascade delete on user model
        $this->deleteUser->handle($hirer->user);

        return $this->respondSuccess("Hirer deleted successfully!");
    }
}
