<?php

namespace App\Http\Controllers\Api\V1;

use App\Http\Controllers\Controller;
use App\Http\Requests\JobRequest;
use App\Http\Resources\JobResource;
use App\Models\Job;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Gate;

class JobController extends Controller
{
    protected array $relations = [
        'employer', 'workType', 'workHour', 'applicants', 'ratings'
    ];

    /**
     * Display a listing of the jobs.
     */
    public function index(): JsonResponse {
        $jobs = $this->paginateOrGet(Job::filter()->latest());

        return $this->respondWithResourceCollection(
            JobResource::collection($jobs)
        );
    }

    /**
     * Store a newly created job in storage.
     */
    public function store(JobRequest $request): JsonResponse {
        $job = Job::create($request->validated());

        return $this->respondCreated(
            new JobResource($job),
            "Job created successfully!"
        );
    }

    /**
     * Display the specified job.
     */
    public function show(Job $job): JsonResponse {
        $job->loadMissing($this->relations);

        return $this->respondWithResource(new JobResource($job));
    }

    /**
     * Update the specified job in storage.
     */
    public function update(JobRequest $request, Job $job): JsonResponse {
        $job->update($request->validated());

        return $this->respondUpdated(
            new JobResource($job),
            "Job updated successfully!"
        );
    }

    /**
     * Remove the specified job from storage.
     */
    public function destroy(Job $job): JsonResponse {
        $job->delete();

        return $this->respondSuccess("Job deleted successfully!");
    }

    public function apply(Request $request, Job $job) {
        Gate::authorize("apply-job");

        $job->applicants()->attach($request->user()->id, ["applied_at" => now()]);

        return $this->respondSuccess(
            "Successfully applied for job!"
        );
    }
}

