<?php

namespace App\Http\Controllers\Api\V1;

use App\Http\Controllers\Controller;
use App\Http\Requests\ServiceRequest;
use App\Http\Resources\ServiceResource;
use App\Models\Service;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Gate;

class ServiceController extends Controller
{
    protected array $relations = [
        'users'
    ];

    /**
     * Display a listing of the services.
     */
    public function index(): JsonResponse {
        $services = $this->paginateOrGet(Service::filter()->latest());

        return $this->respondWithResourceCollection(
            ServiceResource::collection($services)
        );
    }

    /**
     * Store a newly created service in storage.
     */
    public function store(ServiceRequest $request): JsonResponse {
        $service = Service::create($request->validated());

        return $this->respondCreated(
            new ServiceResource($service),
            'Service created successfully!'
        );
    }

    /**
     * Display the specified service.
     */
    public function show(Service $service): JsonResponse {
        $service->loadMissing($this->relations);

        return $this->respondWithResource(new ServiceResource($service));
    }

    /**
     * Remove the specified service from storage.
     */
    public function destroy(Service $service): JsonResponse {
        $service->delete();

        return $this->respondSuccess('Service deleted successfully!');
    }

    public function apply(Request $request, Service $service) {
        $validated = $request->validate([
            "application_date" => "date",
            "status" => 'required|string|max:50',
            'is_paid' => 'required|boolean'
        ]);

        Gate::authorize('apply-service');

        $service->users()->attach($request->user()->id, $validated);

        return $this->respondSuccess(
            "Successfully applied for the service");
    }

    // for multiple service applications
    public function updateApplication(Request $request, Service $service, string $applicationId) {
        $validated = $request->validate([
            "application_date" => "date",
            "status" => 'required|string|max:50',
            'is_paid' => 'required|boolean'
        ]);

        $user = $request->user();

        // get service application
        $application = $user->services()->where('id', $applicationId)->first();

        // check if the user has applied for this service
        if (!$application) {
            return $this->respondNotFound("Service application not found!");
        }

        // update the pivot table (service_user) with the new data
        $application->update($validated);

        return $this->respondSuccess("service application updated successfully!");
    }

    // for single service application
//    public function updateApplication(Request $request, Service $service)
//    {
//        // Validate the request data
//        $validated = $request->validate([
//            'application_date' => ['required', 'date'],
//            'status' => ['nullable', 'string', 'max:50'],
//            'is_paid' => ['required', 'boolean'],
//        ]);
//
//        // Get the currently authenticated user
//        $user = $request->user();
//
//        // Check if the user has applied for this service
//        if (!$user->services()->where('service_id', $service->id)->exists()) {
//            return $this->respondError('No application found for this service.', null, 400);
//        }
//
//        // Update the pivot table (service_user) with the new data
//        $user->services()->updateExistingPivot($service->id, [
//            'application_date' => $validated['application_date'],
//            'status' => $validated['status'],
//            'is_paid' => $validated['is_paid'],
//        ]);
//
//        // Return a success response
//        return response()->json(['message' => 'Application updated successfully.']);
//    }


    /**
     * Update the specified service in storage.
     */
    public function update(ServiceRequest $request, Service $service): JsonResponse {
        $service->update($request->validated());

        return $this->respondUpdated(
            new ServiceResource($service),
            'Service updated successfully!'
        );
    }
}

