<?php

namespace App\Http\Controllers\Api\V1;

use App\Actions\User\CreateUser;
use App\Actions\User\DeleteUser;
use App\Actions\User\UpdateUser;
use App\Http\Controllers\Controller;
use App\Http\Requests\UserRequest;
use App\Http\Requests\WorkerRequest;
use App\Http\Resources\WorkerResource;
use App\Models\Worker;
use Illuminate\Http\JsonResponse;

class WorkerController extends Controller
{
    protected array $fileFieldNames = [
        'aadhar_front', 'aadhar_back', 'pan_card'
    ];

    protected array $fileFolderPaths = [
        'aadhar-fronts', 'aadhar-backs', 'pan-cards'
    ];

    protected array $relations = [
        "user", 'workType', 'agency', 'languages', 'workHours', 'skills'
    ];

    public function __construct(
        protected CreateUser $createUser,
        protected UpdateUser $updateUser,
        protected DeleteUser $deleteUser,
    ) {}


    /**
     * Display a listing of the workers.
     */
    public function index(): JsonResponse {
        $workers = $this->paginateOrGet(Worker::filter()->latest());

        return $this->respondWithResourceCollection(
            WorkerResource::collection($workers)
        );
    }

    /**
     * Store a newly created worker in storage.
     */
    public function store(UserRequest $userRequest, WorkerRequest $workerRequest): JsonResponse {
        // using database transaction
        $worker = $this->transactional(function () use ($userRequest, $workerRequest) {
            // create user and save in the database
            $user = $this->createUser->worker($userRequest);

            // create worker model (not stored in database yet)
            $worker = $user->worker()->make($workerRequest->validated());

            // upload files
            $this->storeFiles($workerRequest, $worker);
            // save worker in the database
            $worker->save();

            // attach relationships: work hours, skills, and languages
            $this->manageWorkerRelations($worker, $workerRequest);

            return $worker;
        });

        return $this->respondCreated(
            new WorkerResource($worker->loadMissing('user')),
            "Worker created successfully!"
        );
    }

    private function manageWorkerRelations(Worker $worker, WorkerRequest $workerRequest, string $method = 'attach') {
        // validate that $method is either 'attach' or 'sync'
        if (!in_array($method, ['attach', 'sync'])) {
            throw new \InvalidArgumentException(
                "Invalid method: $method. Allowed methods are 'attach' or 'sync'.");
        }

        // use the method (attach or sync) for work hours, skills, and languages to attach or sync relation data
        if ($workerRequest->filled('work_hours')) {
            $worker->workHours()->$method($workerRequest->validated('work_hours', []));
        }

        if ($workerRequest->filled('skills')) {
            $worker->skills()->$method($workerRequest->validated('skills', []));
        }

        if ($workerRequest->filled('languages')) {
            $worker->languages()->$method(
                collect($workerRequest->validated('languages', []))->mapWithKeys(fn($lang) => [
                    $lang['id'] => [
                        'can_read' => $lang['can_read'],
                        'can_write' => $lang['can_write'],
                        'can_speak' => $lang['can_speak']
                    ]
                ])
            );
        }
    }

    /**
     * Display the specified worker.
     */
    public function show(Worker $worker): JsonResponse {
        $worker->loadMissing($this->relations);

        return $this->respondWithResource(
            new WorkerResource($worker));
    }

    /**
     * Update the specified worker in storage.
     */
    public function update(UserRequest $userRequest, WorkerRequest $workerRequest, Worker $worker): JsonResponse {
        // using database transaction
        $this->transactional(function () use ($userRequest, $workerRequest, $worker) {
            // update user
            $this->updateUser->handle($userRequest, $worker->user);

            // update worker model (not updated in database yet)
            $worker->fill($workerRequest->validated());

            // update files if provided
            $this->updateFiles($workerRequest, $worker);
            // save updated data in the database
            $worker->save();

            // sync relationships: work hours, skills, and languages
            $this->manageWorkerRelations($worker, $workerRequest, 'sync');
        });

        return $this->respondUpdated(
            new WorkerResource($worker->loadMissing('user')),
            "Worker updated successfully!"
        );
    }

    /**
     * Remove the specified worker from storage.
     */
    public function destroy(Worker $worker): JsonResponse {
        $this->deleteFiles($worker);
        // worker will be deleted automatically because of cascade delete on user model
        $this->deleteUser->handle($worker->user);

        return $this->respondSuccess("Worker deleted successfully!");
    }
}
